#!/bin/bash
#
# Script to split up a song and approximate it with an increasing
# number of voices.
#
# Use ffmpeg to convert the audio into a simple format so we can
# just fread the audio signal.
#
# Neil Birkbeck. 

length=${length-20}

if [ -z $1 ]; then echo "Provide input filename"; exit 1; fi


# Split the song into a bunch of equally spaced parts so we 
# can ramp up the number of voices played overtop.
split_song() {
    start=0
    num=0

    while true;
    do
	echo "Splitting sone ${num} ${start}";
	out=/tmp/split_`printf %04d.raw ${num}`
	ffmpeg -t ${length} -ss ${start} -i "$1" -ac 1 \
	    -ar 22050 -ac 1 -vn -f f32le -acodec pcm_f32le \
	    ${out}
	    
	
	if [[ ! -s ${out} ]];
	then
	    break;
	fi

	start=$((start+length))
	num=$((num+1))
    done
}

# Run each of the parts, r
run_parts() {
    readonly NUMPROC=4
    
    # Start with a certain number of iterations and increase by a factor of 2 
    # each time.
    its=5
    i=0
    for file in /tmp/split_*.raw;
    do
	echo Running $file
	./a.out --db /tmp/Sounds --probe "$file" \
	    --out ${file/split/out} --its=${its}> $file.log&
	i=$((i+1))

	if [[ $((i%NUMPROC)) = 0 ]]; 
	then 
	    echo "Waiting for jobs $i, last ${file}";
	    wait; 
	fi
	its=$((its*2))
    done
    wait
}

# Merge all the resulting parts into the final result.
merge_parts() {
    cat /tmp/out_*.raw > /tmp/out.raw
    ffmpeg -ar 22050 -ac 1 -f f32le -acodec pcm_f32le \
	-i /tmp/out.raw /tmp/out.wav
}


# Kill all processess on SIGINT.
trap "kill 0" SIGINT

# main()
rm -fv /tmp/split_*.raw
split_song "$1"
run_parts "$1"
merge_parts "$1"
